%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(queue_SUITE).
-export([all/1]).

-export([do/1, to_list/1]).

-export([init_per_testcase/2, fin_per_testcase/2]).

-include("test_server.hrl").

% Default timetrap timeout (set in init_per_testcase).
-define(default_timeout, ?t:minutes(1)).

init_per_testcase(Case, Config) ->
    ?line Dog = ?t:timetrap(?default_timeout),
    [{watchdog, Dog} | Config].
fin_per_testcase(Case, Config) ->
    Dog = ?config(watchdog, Config),
    test_server:timetrap_cancel(Dog),
    ok.

all(doc) ->
    ["Test cases for queue."];
all(suite) ->
    [do, to_list].

do(doc) ->
    [""];
do(suite) ->
    [];
do(Config) when list(Config) ->
    ?line E = queue:new(),
    ?line [] = queue:to_list(E),

    ?line do_queue(E,
		   [{in, 1},
		    {in, 2},
		    {out, {value, 1}},
		    {in, 3},
		    {out, {value, 2}},
		    {out, {value, 3}},
		    {out, empty}
		   ]),
    ok.

to_list(doc) ->
    ["OTP-2701"];
to_list(suite) ->
    [];
to_list(Config) when list(Config) ->
    ?line E = queue:new(),
    ?line Q = do_queue(E, [{in, 1},
			   {in, 2},
			   {in, 3},
			   {out, {value, 1}},
			   {in, 4},
			   {in, 5}]),
    ?line case queue:to_list(Q) of
	      [2,3,4,5] ->
		  ok;
	      Other ->
		  test_server:fail(Other)
	  end,

    ok.

do_queue(Q, []) ->
    Q;
do_queue(Q, [E | Rest]) ->
    do_queue(do_queue_1(E, Q), Rest).

do_queue_1({in, E}, Q) ->
    queue:in(E, Q);
do_queue_1({out, E}, Q) ->
    case queue:out(Q) of
	{E, Q1} ->
	    Q1;
	Other ->
	    test_server:fail({"out failed", E, Q, Other})
    end.

	    
