%% ``The contents of this file are subject to the Erlang Public License,
%% Version 1.1, (the "License"); you may not use this file except in
%% compliance with the License. You should have received a copy of the
%% Erlang Public License along with this software. If not, it can be
%% retrieved via the world wide web at http://www.erlang.org/.
%% 
%% Software distributed under the License is distributed on an "AS IS"
%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%% the License for the specific language governing rights and limitations
%% under the License.
%% 
%% The Initial Developer of the Original Code is Ericsson Utvecklings AB.
%% Portions created by Ericsson are Copyright 1999, Ericsson Utvecklings
%% AB. All Rights Reserved.''
%% 
%%     $Id$
%%
-module(supervisor_1).

-export([start_child/1, start_child/2, init/1]).

-export([handle_call/3, handle_info/2, terminate/2]).


start_child(Tester) ->
    gen_server:start_link(?MODULE, {normal, Tester}, []).
start_child(Tester, error) ->
    case get(start_child_error) of
	undefined ->
	    put(start_child_error, set),
	    start_child(Tester);
	set -> gen_server:start_link(?MODULE, {error, Tester}, [])
    end;

start_child(Tester, Extra) ->
    {ok, Pid} = gen_server:start_link(?MODULE, {normal, Tester}, []),
    {ok, Pid, Extra}.

init({normal, Tester}) ->
    process_flag(trap_exit, true),
    {ok, Tester}.

handle_call(Req, From, State) ->
    {reply, Req, State}.

handle_info(die, State) ->
    {stop, died, State};
handle_info(_, State) ->
    {noreply, State}.

terminate(Reason, Tester) ->
    Tester ! {child_die, self(), Reason},
    ok.

